﻿Param(
[string]$CAPA_Username,
[string]$CAPA_Password,
[string]$CAPA_ServerName,
[string]$CAPA_InstanceID
)


###############################################################################################################################################################
# Start Function: GetCAPA_DataSet
###############################################################################################################################################################
function GetCAPA_DataSet ($CAPA_DataSet_Criteria)
{ 
    $messages = @()

    try
    {
        $PWord = ConvertTo-SecureString -String $CAPA_Password -AsPlainText -Force;
        $CAPA_Credential = New-Object -TypeName System.Management.Automation.PSCredential -ArgumentList $CAPA_Username, $PWord;
        $headers = @{}
        $headers["Accept"] = "application/json"

        if($debugOn) { $messages += "Reading CA PA Process Dataset for PID: $($CAPA_InstanceID)" }
        $CAPA_URL = "https://$($CAPA_ServerName)/api/rest/v1/processinstances/$($CAPA_InstanceID)/processdataset"
        $query_result = Invoke-RestMethod -Uri $CAPA_URL -Credential $CAPA_Credential -UseBasicParsing -Headers $headers
        $local_dataset = $query_result.Response.Details.ProcessDataset.Parameters.Param | Where-Object Name -Like $CAPA_DataSet_Criteria

        if($local_dataset -eq $null)
        {
                $messages += "Criteria $($CAPA_DataSet_Criteria) returned 0 matches"
                $resultValue = New-Object PSObject
                $resultValue | add-member Noteproperty status "failure"
                $resultValue | add-member Noteproperty messages $messages -Force
                return $resultValue
        }

        if($debugOn) { $messages += "Criteria $($CAPA_DataSet_Criteria) matched." }
        $resultsDictionary = New-Object 'system.collections.generic.dictionary[string,string]'
        $local_dataset = $local_dataset | Where-Object Type -ne "Password"
        $local_dataset | ForEach { 
            $resultsDictionary[$_.Name] = $_.Value 
        }

        $datasetPointers = $local_dataset | Where-Object Value -Like 'DataSet*'
        ForEach($datasetPointer in $datasetPointers)
        {
            try
            {
                $getPointerValue = $datasetPointer.Value
                $pointerLocationValue = $getPointerValue.ToString().Replace("Datasets[""/Common/DataSets/","").Replace("""]","")
                $pointerValue = $resultsDictionary[$pointerLocationValue]
                $resultsDictionary[$datasetPointer.Name] = $pointerValue
            }
            catch
            {
                $resultsDictionary[$datasetPointer.Name] = $null
            }
        }
        if($debugOn) { $messages += "Dictionary was created successfully." }

        $resultValue = New-Object PSObject
        $resultValue | add-member Noteproperty status "success" -Force
        $resultValue | add-member Noteproperty messages $messages -Force
        $resultValue | add-member Noteproperty results $resultsDictionary -Force
        return $resultValue
    }
    catch
    {
        $resultValue = New-Object PSObject
        $resultValue | add-member Noteproperty status "failure" -Force
        $messages += "Caught Error: $($_.Exception.Message)"
        $messages += "Error located at $($_.InvocationInfo.ScriptLineNumber),$($_.InvocationInfo.OffsetInLine)"

        if($_.Exception.Message -eq "The remote server returned an error: (500) Internal Server Error.")
        {
            $errorResult = $_.Exception.Response.GetResponseStream()
            $errorReader = New-Object System.IO.StreamReader($errorResult)
            $errorReader.BaseStream.Position = 0
            $errorReader.DiscardBufferedData()
            $errorResponseBody = $errorReader.ReadToEnd();

            if($errorResponseBody.IndexOf("Error: Runtime instance with ID:") -ne -1)
            {
                $messages = ""
                $messages += "Runtime instance matching $($CAPA_InstanceID) not found!"
                $resultValue | add-member Noteproperty messages $messages -Force
                return $resultValue
            }
            else
            {
                $resultValue | add-member Noteproperty messages $messages -Force
                return $resultValue
            }
        }
        else
        {
            $resultValue | add-member Noteproperty messages $messages -Force
            return $resultValue
        }
    }
}
###############################################################################################################################################################
# End Function
###############################################################################################################################################################

###############################################################################################################################################################
# Start Function: ReturnDataSetVariable
###############################################################################################################################################################
function ReturnDataSetVariable($CheckVariable)
{
    try
    {
        $checkReturn = @()
        $checkReturn += ($global:DataSet.GetEnumerator() | Where-Object { $_.Key -like $CheckVariable})
        if($checkReturn.Count -eq 1)
        {

            return $checkReturn[0].Value
        }
        else
        {
            return $checkReturn 
        }
    }
    catch
    {
        return $null
    }
}
###############################################################################################################################################################
# End Function
###############################################################################################################################################################


###############################################################################################################################################
# Start Main
###############################################################################################################################################
try
{
    
    if([string]::IsNullOrEmpty($CAPA_InstanceID) -eq $true)
    {
        $messages += "No CAPA Instance ID was passed."
        $resultValue = New-Object PSObject
        $resultValue | add-member Noteproperty status "failure"
        $resultValue | add-member Noteproperty messages $messages
	    $resultValue | Add-Member Noteproperty results $null -Force
        $output = $resultValue | ConvertTo-Json -Depth 100
        Write-Host $output
        Exit 1
    }

    #Query the CA PA Process Dataset for the passed PID
    $capaDataSet = GetCAPA_DataSet "*"
    if($capaDataSet.status.ToUpper() -ne "SUCCESS")
    {
        $resultValue = New-Object PSObject
        $resultValue | add-member Noteproperty status "failure"
        $resultValue | add-member Noteproperty messages $capaDataSet.messages
	    $resultValue | Add-Member Noteproperty results $null -Force
        $output = $resultValue | ConvertTo-Json -Depth 100
        Write-Host $output
        Exit 1
    }
    $global:DataSet = $capaDataSet.results
    $global:capaEnvironment = "Development"

    #Return a list of all the variables
    $global:DataSet | ft

    ReturnDataSetVariable "*Environment.SCOM.Preproduction*"
    
}
catch
{
    $resultValue = New-Object PSObject
    $resultValue | add-member Noteproperty status "failure"  -Force
    $messages += "Caught Error: $($_.Exception.Message)"
    $messages += "Error located at $($_.InvocationInfo.ScriptLineNumber),$($_.InvocationInfo.OffsetInLine)"
    $resultValue = New-Object PSObject
    $resultValue | add-member Noteproperty status "failure" -Force
    $resultValue | add-member Noteproperty messages $messages -Force
    $output = $resultValue | ConvertTo-Json -Depth 100
    Write-Host $output
    Exit 1
}
###############################################################################################################################################
# End Main
###############################################################################################################################################